#! /usr/bin/python
#
#     By: Jim Kemp
#     Powerhouse Electronics 
#     ph-elec.com
#     10/26/2009
#
# The following code was addated from Scott Dattalo web site. Scott provided 
# some assembly routines to calulate CRC-8 on a Pic. I rewrote these assembly
# rotines into Python code.
# http://www.dattalo.com/technical/software/pic/crc_8.asm
#
# ==============================================================================
def crc8( lst ):
    """
    Given a list of numbers, calculate the CRC-8 value.
    """
    crc = 0
    for dat in lst:
        crc ^= dat

        dat = 0x3B
        if crc & 0x01: dat ^= 0x1D
        if crc & 0x02: dat ^= 0x3A
        if crc & 0x04: dat ^= 0x74
        if crc & 0x08: dat ^= 0xE8
        if crc & 0x10: dat ^= 0xCD
        if crc & 0x20: dat ^= 0x87
        if crc & 0x40: dat ^= 0x13
        if crc & 0x80: dat ^= 0x26
        crc = dat

    return crc


# Perform some testing on crc8 to make sure she works!
# ==============================================================================
if __name__ == '__main__':
    print "CRC Zero test: 3B BE F1 59"
    print "CRC 1: 0x%X" % crc8( [0] )
    if crc8( [0] ) != 0x3B: print "Err"
    print "CRC 2: 0x%X" % crc8( [0,0] )
    if crc8( [0,0] ) != 0xBE: print "Err"
    print "CRC 3: 0x%X" % crc8( [0,0,0] )
    if crc8( [0,0,0] ) != 0xF1: print "Err"
    print "CRC 4: 0x%X" % crc8( [0,0,0,0] )
    if crc8( [0,0,0,0] ) != 0x59: print "Err"

    print

    print "CRC FF test: FF 3B 7A 74"
    print "CRC 1: 0x%X" % crc8( [0xFF] )
    if crc8( [0xFF] ) != 0xFF: print "Err"
    print "CRC 2: 0x%X" % crc8( [0xFF,0xFF] )
    if crc8( [0xFF, 0xFF] ) != 0x3B: print "Err"
    print "CRC 3: 0x%X" % crc8( [0xFF,0xFF,0xFF] )
    if crc8( [0xFF, 0xFF, 0xFF] ) != 0x7A: print "Err"
    print "CRC 4: 0x%X" % crc8( [0xFF,0xFF,0xFF,0xFF] )
    if crc8( [0xFF, 0xFF, 0xFF, 0xFF] ) != 0x74: print "Err"



    
