#! /usr/bin/python

Pterm = 2.0  ; P = 0
Iterm = 0.01 ; I = 0
Dterm = 1.00 ; D = 0

# Sets the time span (10 seconds) the derivative is calulated over. 
len = 10
rate = 0.0
Tobj = 0

false = 0; true = 1

# ==============================================================================
def myPID( var, setpt, output, man_auto_sw = false ):
    """
    Super Simple PID algorithm.

    Call this function once per second. Set man_auto_sw true to make a bumpless
    switch from manual to automatic mode.
    """

    global Pterm, Iterm, Dterm, P, I, D, E, T, rate, len

    # ==========================================================================
    def clamp( min, tmp, max ):
        if tmp > max: return max
        if tmp < min: return min
        return tmp

    # Calculate error term.
    err = var - setpt

    # Calculate proportional term.
    P = err * Pterm * -1.0
    P = clamp( -100.0, P, 100.0 )

    if man_auto_sw:
        # Jump I to value which will make for a bumpless transfer
        # from manual to auto.
        I = output - P
    else:
        # Only start integrating when the rate of change is small.
        if (D > -10) and (D < 10):
            # Calculate integral term - Integrate err.
            I = I + clamp( -0.1, err * Iterm * -1.0, 0.1 )      
            # Don't let I wind up.
            I = clamp( -10, I, 60 )   # Keep I in the range - prevent winding.
        # Else, drive the I term to zero.
        else:
            if I < 0: I = I + Iterm   # Slowly unwind I down to zero.
            else:     I = I - Iterm 

    # Calulate the rate of change.
    del T[0]                      # Delete oldest value out of list.
    T.append( var )               # Add new temp onto end of list.
    rate = T[ len-1 ] - T[0]      # Calulate the rate of change over 10 seconds.

    # Derivative Logic
    del E[0]
    E.append( err )
    rateErr = E[ len-1 ] - E[0]

    D = rateErr * Dterm * -1
    D = clamp( -40, D, 40 )

    # Only drive the output if in auto mode.
    if man_auto_sw == false:
        # Set output output as a percentage.
        output = clamp( 0, P + I + D, 100 )

    return output, rate


# Test the PID here.
# ==============================================================================
if __name__ == '__main__':

    from time import sleep
    from myStreamReader import myReader  # Use actual temperatures for testing.

    setpt = 50  # Lets see if we can control to 50 degC as a test.
    output = 20
    i = 1
    dir = 1

    # Call reader until it gets synced.
    while Tobj == 0:
        Tobj, Tamb = myReader( output )
    
    E = [0] * len               # Init the list to all zeros.
    T = [Tobj] * len            # Keep a list of temperatures too.

    while 1:
        # The stream reader must be called at 10Hz.
        Tobj, Tamb = myReader( output )

        # Once per second perform the PID stuff.
        if (i % 10) == 0:
            # Set the output to 1%. # Output should come on for 50ms every 4 seconds.
            auto = true
            output, rate = myPID( Tobj, setpt, output, man_auto_sw = auto )
            print "%3d P=%4.1f I=%+.1f D=%+.1f   Obj: %.1f Amb: %.1f Rate: %.1f SetPt: %d" % \
                (output, P, I, D, Tobj, Tamb, rate, setpt)
        i += 1
        sleep( 0.1 )

